'''render/audio.py'''

from __future__ import division

import os

from auto_editor.utils.func import get_new_length, fnone
from auto_editor.utils.progressbar import ProgressBar
from auto_editor.scipy.wavfile import read, write

def convert_audio(ffmpeg, input_path, inp, output_path, codec, log):
    if(fnone(codec)):
        codec = inp.audio_streams[0]['codec']

    ffmpeg.run(['-i', input_path, '-acodec', codec, output_path])

def handle_audio(ffmpeg, input_path, bitrate, samplerate, temp, log):
    # type: (...) -> str
    temp_file = os.path.join(temp, 'faAudio.wav')

    log.checkType(samplerate, 'samplerate', str)
    cmd = ['-i', input_path]
    if(not fnone(bitrate)):
        cmd.extend(['-b:a', bitrate])
    cmd.extend(['-ac', '2', '-ar', samplerate, '-vn', temp_file])

    ffmpeg.run(cmd)
    log.conwrite('')

    return temp_file

def make_new_audio(input_path, output_path, chunks, speeds, log, fps, machine_readable,
    hide_bar):
    import numpy as np

    def custom_speeds(a):
        # type: (list[int | float]) -> bool
        return len([x for x in a if x != 1 and x != 99999]) > 0

    if(custom_speeds(speeds)):
        from audiotsm2 import phasevocoder
        from audiotsm2.io.array import ArrReader, ArrWriter

    if(len(chunks) == 1 and chunks[0][2] == 0):
        log.error('Trying to create an empty file.')

    samplerate, audio_samples = read(input_path)

    # Get the new length in samples with some extra leeway.
    new_length = get_new_length(chunks, speeds, fps)
    estimated_length = int(new_length * samplerate * 1.5) + int(samplerate * 2)

    # Create an empty array for the new audio.
    new_audio = np.zeros((estimated_length, 2), dtype=np.int16)

    channels = 2
    y_pointer = 0

    progress = ProgressBar(len(chunks), 'Creating new audio', machine_readable,
        hide_bar)

    for c, chunk in enumerate(chunks):
        sample_start = int(chunk[0] / fps * samplerate)
        sample_end = int(sample_start + (samplerate / fps) * (chunk[1] - chunk[0]))

        the_speed = speeds[chunk[2]]
        if(the_speed != 99999):
            sped_chunk = audio_samples[sample_start:sample_end]

            if(the_speed == 1):
                y_end_pointer = y_pointer + sped_chunk.shape[0]
                new_audio[y_pointer:y_end_pointer] = sped_chunk
            else:
                spedup_audio = np.zeros((0, 2), dtype=np.int16)
                with ArrReader(sped_chunk, channels, samplerate, 2) as reader:
                    with ArrWriter(spedup_audio, channels, samplerate, 2) as writer:
                        phasevocoder(reader.channels, speed=the_speed).run(
                            reader, writer
                        )
                        spedup_audio = writer.output

                y_end_pointer = y_pointer + spedup_audio.shape[0]
                new_audio[y_pointer:y_end_pointer] = spedup_audio

            my_samples = ((chunk[1] - chunk[0]) / fps) * samplerate
            new_samples = int(my_samples / the_speed)

            y_pointer = y_pointer + new_samples
        else:
            # Speed is too high so skip this section.
            y_end_pointer = y_pointer

        progress.tick(c)

    log.debug('Total Samples: {}'.format(y_pointer))
    log.debug('Samples per Frame: {}'.format(samplerate / fps))
    log.debug('Expected video length: {}'.format(y_pointer / (samplerate / fps)))
    new_audio = new_audio[:y_pointer]
    write(output_path, samplerate, new_audio)

    log.conwrite('')



'''utils/video.py'''

# Internal Libraries
import os
from shutil import move

# Included Libraries
from .func import fnone
from auto_editor.render.audio import make_new_audio

def handle_audio_tracks(ffmpeg, write_file, args, a_tracks, chunks, speeds, fps, temp,
    log):
    for t in range(a_tracks):
        temp_file = os.path.join(temp, '{}.wav'.format(t))
        new_file = os.path.join(temp, 'new{}.wav'.format(t))
        make_new_audio(temp_file, new_file, chunks, speeds, log, fps,
            args.machine_readable_progress, args.no_progress)

        if(not os.path.isfile(new_file)):
            log.bug('Audio file not created.')

    if(args.export_as_audio):
        if(args.keep_tracks_seperate):
            log.warning("Audio files don't have multiple a_tracks.")

        if(a_tracks == 1):
            move(os.path.join(temp, '0.wav'), write_file)
            return False

        cmd = []
        for t in range(a_tracks):
            cmd.extend(['-i', os.path.join(temp, '{}.wav'.format(t))])
        cmd.extend(['-filter_complex', 'amix=inputs={}:duration=longest'.format(a_tracks),
            write_file])
        ffmpeg.run(cmd)

        return False
    return True

def mux_rename_video(ffmpeg, spedup, write_file, args, inp, temp, log):

    s_tracks = len(inp.subtitle_streams)
    a_tracks = len(inp.audio_streams)

    if(a_tracks == 0 and s_tracks == 0):
        move(spedup, write_file)
        log.conwrite('')
        return

    cmd = ['-i', spedup]
    if(a_tracks > 0):
        if(args.keep_tracks_seperate):
            for t in range(a_tracks):
                cmd.extend(['-i', os.path.join(temp, 'new{}.wav'.format(t))])
        else:
            # Merge all the audio a_tracks into one.
            new_a_file = os.path.join(temp, 'new_audio.wav')
            if(a_tracks > 1):
                new_cmd = []
                for t in range(a_tracks):
                    new_cmd.extend(['-i', os.path.join(temp, 'new{}.wav'.format(t))])
                new_cmd.extend(['-filter_complex', 'amerge=inputs={}'.format(a_tracks),
                    '-ac', '2', new_a_file])
                ffmpeg.run(new_cmd)
            else:
                new_a_file = os.path.join(temp, 'new0.wav')
            cmd.extend(['-i', new_a_file])

    if(s_tracks > 0):
        for s, sub in enumerate(inp.subtitle_streams):
            new_path = os.path.join(temp, 'new{}s.{}'.format(s, sub['ext']))
            cmd.extend(['-i', new_path])

    total_streams = 1 + s_tracks + (a_tracks if args.keep_tracks_seperate else 1)
    for i in range(total_streams):
        cmd.extend(['-map', '{}:0'.format(i)])

    cmd.extend(['-c:v', 'copy'])
    if(s_tracks > 0):
        codec = inp.subtitle_streams[0]['codec']
        cmd.extend(['-c:s', codec])

    if(not fnone(args.audio_codec)):
        cmd.extend(['-c:a', args.audio_codec])

    cmd.append(write_file)
    ffmpeg.run(cmd)
    log.conwrite('')
